package com.ruoyi.business.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.service.IHwSceneService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.business.domain.HwAlarmLevel;
import com.ruoyi.business.service.IHwAlarmLevelService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.page.TableDataInfo;

/**
 * 报警级别Controller
 * 
 * @author xins
 * @date 2023-09-15
 */
@RestController
@RequestMapping("/alarmLevel")
public class HwAlarmLevelController extends BaseController
{
    @Autowired
    private IHwAlarmLevelService hwAlarmLevelService;

    @Autowired
    private IHwSceneService hwSceneService;

    /**
     * 查询报警级别列表
     */
    @RequiresPermissions("business:alarmLevel:list")
    @GetMapping("/list")
    public TableDataInfo list(HwAlarmLevel hwAlarmLevel)
    {
        startPage();
        List<HwAlarmLevel> list = hwAlarmLevelService.selectHwAlarmLevelJoinList(hwAlarmLevel);
        return getDataTable(list);
    }

    /**
     * 导出报警级别列表
     */
    @RequiresPermissions("business:alarmLevel:export")
    @Log(title = "报警级别", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwAlarmLevel hwAlarmLevel)
    {
        List<HwAlarmLevel> list = hwAlarmLevelService.selectHwAlarmLevelJoinList(hwAlarmLevel);
        ExcelUtil<HwAlarmLevel> util = new ExcelUtil<HwAlarmLevel>(HwAlarmLevel.class);
        util.exportExcel(response, list, "报警级别数据");
    }

    /**
     * 获取报警级别详细信息
     */
    @RequiresPermissions("business:alarmLevel:query")
    @GetMapping(value = "/{alarmLevelId}")
    public AjaxResult getInfo(@PathVariable("alarmLevelId") Long alarmLevelId)
    {
        return success(hwAlarmLevelService.selectHwAlarmLevelByAlarmLevelId(alarmLevelId));
    }

    /**
     * 新增报警级别
     */
    @RequiresPermissions("business:alarmLevel:add")
    @Log(title = "报警级别", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwAlarmLevel hwAlarmLevel)
    {
        return toAjax(hwAlarmLevelService.insertHwAlarmLevel(hwAlarmLevel));
    }

    /**
     * 修改报警级别
     */
    @RequiresPermissions("business:alarmLevel:edit")
    @Log(title = "报警级别", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwAlarmLevel hwAlarmLevel)
    {
        return toAjax(hwAlarmLevelService.updateHwAlarmLevel(hwAlarmLevel));
    }

    /**
     * 删除报警级别
     */
    @RequiresPermissions("business:alarmLevel:remove")
    @Log(title = "报警级别", businessType = BusinessType.DELETE)
	@DeleteMapping("/{alarmLevelId}")
    public AjaxResult remove(@PathVariable Long alarmLevelId)
    {
        return toAjax(hwAlarmLevelService.deleteHwAlarmLevelByAlarmLevelId(alarmLevelId));
    }

    @RequiresPermissions("business:alarmLevel:list")
    @GetMapping("/getScenes")
    public AjaxResult getScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList(scene));
    }

    @RequiresPermissions("business:alarmLevel:list")
    @GetMapping("/getEditedScenes")
    public AjaxResult getEditedScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList4Select(scene));
    }

}
