package com.ruoyi.business.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.service.IHwSceneService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.business.domain.HwAlarmType;
import com.ruoyi.business.service.IHwAlarmTypeService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.page.TableDataInfo;

/**
 * 报警类型Controller
 * 
 * @author xins
 * @date 2023-09-15
 */
@RestController
@RequestMapping("/alarmType")
public class HwAlarmTypeController extends BaseController
{
    @Autowired
    private IHwAlarmTypeService hwAlarmTypeService;

    @Autowired
    private IHwSceneService hwSceneService;

    /**
     * 查询报警类型列表
     */
    @RequiresPermissions("business:alarmType:list")
    @GetMapping("/list")
    public TableDataInfo list(HwAlarmType hwAlarmType)
    {
        startPage();
        List<HwAlarmType> list = hwAlarmTypeService.selectHwAlarmTypeJoinList(hwAlarmType);
        return getDataTable(list);
    }

    /**
     * 导出报警类型列表
     */
    @RequiresPermissions("business:alarmType:export")
    @Log(title = "报警类型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwAlarmType hwAlarmType)
    {
        List<HwAlarmType> list = hwAlarmTypeService.selectHwAlarmTypeJoinList(hwAlarmType);
        ExcelUtil<HwAlarmType> util = new ExcelUtil<HwAlarmType>(HwAlarmType.class);
        util.exportExcel(response, list, "报警类型数据");
    }

    /**
     * 获取报警类型详细信息
     */
    @RequiresPermissions("business:alarmType:query")
    @GetMapping(value = "/{alarmTypeId}")
    public AjaxResult getInfo(@PathVariable("alarmTypeId") Long alarmTypeId)
    {
        return success(hwAlarmTypeService.selectHwAlarmTypeByAlarmTypeId(alarmTypeId));
    }

    /**
     * 新增报警类型
     */
    @RequiresPermissions("business:alarmType:add")
    @Log(title = "报警类型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwAlarmType hwAlarmType)
    {
        return toAjax(hwAlarmTypeService.insertHwAlarmType(hwAlarmType));
    }

    /**
     * 修改报警类型
     */
    @RequiresPermissions("business:alarmType:edit")
    @Log(title = "报警类型", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwAlarmType hwAlarmType)
    {
        return toAjax(hwAlarmTypeService.updateHwAlarmType(hwAlarmType));
    }

    /**
     * 删除报警类型
     */
    @RequiresPermissions("business:alarmType:remove")
    @Log(title = "报警类型", businessType = BusinessType.DELETE)
	@DeleteMapping("/{alarmTypeId}")
    public AjaxResult remove(@PathVariable Long alarmTypeId)
    {
        return toAjax(hwAlarmTypeService.deleteHwAlarmTypeByAlarmTypeId(alarmTypeId));
    }

    @RequiresPermissions("business:alarmType:list")
    @GetMapping("/getScenes")
    public AjaxResult getScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList(scene));
    }

    @RequiresPermissions("business:alarmType:list")
    @GetMapping("/getEditedScenes")
    public AjaxResult getEditedScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList4Select(scene));
    }

}
