package com.ruoyi.business.controller;

import com.ruoyi.business.domain.HwDevice;
import com.ruoyi.business.domain.HwDeviceMode;
import com.ruoyi.business.domain.HwMonitorUnit;
import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.domain.VO.DeviceControlVo;
import com.ruoyi.business.domain.VO.HwDeviceImportVo;
import com.ruoyi.business.service.IHwDeviceModeService;
import com.ruoyi.business.service.IHwDeviceService;
import com.ruoyi.business.service.IHwMonitorUnitService;
import com.ruoyi.business.service.IHwSceneService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.InnerAuth;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 设备信息Controller
 *
 * @author xins
 * @date 2023-08-24
 */
@RestController
@RequestMapping("/device")
public class HwDeviceController extends BaseController {
    @Autowired
    private IHwDeviceService hwDeviceService;

    @Autowired
    private IHwMonitorUnitService hwMonitorUnitService;

    @Autowired
    private IHwDeviceModeService hwDeviceModeService;

    @Autowired
    private IHwSceneService hwSceneService;

    /**
     * 查询设备信息列表
     */
    @RequiresPermissions("business:device:list")
    @GetMapping("/list")
    public TableDataInfo list(HwDevice hwDevice) {
        startPage();
        List<HwDevice> list = hwDeviceService.selectHwDeviceJoinList(hwDevice);
        return getDataTable(list);
    }


    /**
     * 导出设备信息列表
     */
    @RequiresPermissions("business:device:export")
    @Log(title = "设备信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwDevice hwDevice) {
        List<HwDevice> list = hwDeviceService.selectHwDeviceJoinList(hwDevice);
        ExcelUtil<HwDevice> util = new ExcelUtil<HwDevice>(HwDevice.class);
        util.exportExcel(response, list, "设备信息数据");
    }


    @Log(title = "设备信息", businessType = BusinessType.IMPORT)
    @RequiresPermissions("business:device:import")
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file, boolean updateSupport, Long sceneId) throws Exception {
        ExcelUtil<HwDeviceImportVo> util = new ExcelUtil<HwDeviceImportVo>(HwDeviceImportVo.class);
        List<HwDeviceImportVo> deviceList = util.importExcel(file.getInputStream());
        String message = hwDeviceService.importDevices(deviceList, updateSupport, sceneId);
        return success(message);
    }

    @RequiresPermissions("business:device:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response, Long sceneId) throws IOException {
        ExcelUtil<HwDeviceImportVo> util = new ExcelUtil<HwDeviceImportVo>(HwDeviceImportVo.class);
        Map<String, List<?>> classListMap = new HashMap<String, List<?>>();

        //根据场景ID获取最子级监控单元列表
        HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
        queryMonitorUnit.setSceneId(sceneId);
        queryMonitorUnit.setMonitorUnitStatus(HwDictConstants.MONITOR_UNIT_STATUS_NORMAL);
        List<HwMonitorUnit> monitorList = hwMonitorUnitService.selectLimitSubMonitorUnit(queryMonitorUnit);
        classListMap.put("HwMonitorUnit", monitorList);

        //根据场景ID获取没有删除的网关设备列表
        HwDevice queryDevice = new HwDevice();
        queryDevice.setSceneId(sceneId);
        queryDevice.setDeviceType(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE);
        List<HwDevice> gatewayDevices = hwDeviceService.selectHwDeviceList(queryDevice);
        classListMap.put("HwDevice", gatewayDevices);

        //根据场景ID获取启用的设备模型列表
        HwDeviceMode queryDeviceMode = new HwDeviceMode();
        queryDeviceMode.setSceneId(sceneId);
        queryDeviceMode.setDeviceModeStatus(HwDictConstants.DEVICE_MODE_STATUS_NORMAL);
        List<HwDeviceMode> deviceModes = hwDeviceModeService.selectHwDeviceModeList(queryDeviceMode);
        classListMap.put("HwDeviceMode", deviceModes);

        util.importTemplateExcel(response, "设备数据", classListMap);
    }


    /**
     * 获取设备信息详细信息
     */
    @RequiresPermissions("business:device:query")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId) {
        return success(hwDeviceService.selectHwDeviceByDeviceId(deviceId));
    }

    /**
     * 新增设备信息
     */
    @RequiresPermissions("business:device:add")
    @Log(title = "设备信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwDevice hwDevice) {
        hwDevice.setTenantId(SecurityUtils.getTenantId());
        hwDevice.setCreateBy(SecurityUtils.getUsername());
        return toAjax(hwDeviceService.insertHwDevice(hwDevice));
    }

    /**
     * 修改设备信息
     */
    @RequiresPermissions("business:device:edit")
    @Log(title = "设备信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwDevice hwDevice) {
        hwDevice.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(hwDeviceService.updateHwDevice(hwDevice));
    }

    /**
     * 删除设备信息
     */
    @RequiresPermissions("business:device:remove")
    @Log(title = "设备信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deviceId}/{deviceType}")
    public AjaxResult remove(@PathVariable("deviceId") Long deviceId
            , @PathVariable("deviceType") String deviceType) {
        if (deviceType.equals(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE)
                && hwDeviceService.checkExistSubDevice(deviceId)) {
            return warn("存在子设备,不允许删除");
        }

        return toAjax(hwDeviceService.deleteHwDeviceByDeviceId(deviceId));
    }

    @RequiresPermissions("business:device:list")
    @GetMapping("/getScenes")
    public AjaxResult getScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList(scene));
    }

    @RequiresPermissions("business:device:list")
    @GetMapping("/getEditedScenes")
    public AjaxResult getEditedScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList4Select(scene));
    }

    /**
     * 获取监控单元树列表
     */
    @RequiresPermissions("business:device:list")
    @GetMapping(value = {"/monitorUnitTree/", "/monitorUnitTree/{sceneId}"})
    public AjaxResult monitorUnitTree(@PathVariable(value = "sceneId", required = false) Long sceneId) {
        HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
        queryMonitorUnit.setSceneId(sceneId);
        return success(hwMonitorUnitService.selectMonitorTreeList(queryMonitorUnit));
    }

    @RequiresPermissions("business:device:list")
    @GetMapping(value = {"/getDeviceModes/", "/getDeviceModes/{sceneId}"})
    public AjaxResult getDeviceModes(@PathVariable(value = "sceneId", required = false) Long sceneId) {
        HwDeviceMode queryDeviceMode = new HwDeviceMode();
        queryDeviceMode.setSceneId(sceneId);
        return success(hwDeviceModeService.selectHwDeviceModeList(queryDeviceMode));
    }

    @RequiresPermissions("business:device:list")
    @GetMapping(value = {"/getGatewayDevices/", "/getGatewayDevices/{sceneId}"})
    public AjaxResult getGatewayDevices(@PathVariable(value = "sceneId", required = false) Long sceneId) {
        HwDevice queryHwDevice = new HwDevice();
        queryHwDevice.setSceneId(sceneId);
        queryHwDevice.setDeviceType(HwDictConstants.DEVICE_TYPE_GATEWAY_DEVICE);
        return success(hwDeviceService.selectHwDeviceList(queryHwDevice));
    }


    /**
     * 状态修改
     */
    @RequiresPermissions("business:device:edit")
    @Log(title = "设备管理", businessType = BusinessType.PUBLISH)
    @PutMapping("/changeDeviceStatus")
    public AjaxResult changeDeviceStatus(@RequestBody HwDevice device) {
        return toAjax(hwDeviceService.changeDeviceStatus(device));
    }

    /**
     * 状态修改
     */
    @RequiresPermissions("business:device:edit")
    @Log(title = "下发设备控制指令", businessType = BusinessType.CONTROL)
    @PutMapping("/publishControlCommand")
    public AjaxResult publishControlCommand(@RequestBody DeviceControlVo deviceControlVo) {
        hwDeviceService.publishControlCommand(deviceControlVo);
        return success();
    }


    @RequiresPermissions("business:device:rebuild")
    @Log(title = "设备信息", businessType = BusinessType.REBUILD)
    @GetMapping("/rebuildTdTables")
    public void rebuildTdTables() {
        hwDeviceService.rebuildTdTables();
    }


    @InnerAuth
    @GetMapping("/computeOnlineDevicecCount/{days}")
    public void computeOnlineDevicecCount(@PathVariable("days") int days) {
        hwDeviceService.computeOnlineDevicecCount(days);
    }

}
