package com.ruoyi.business.controller;

import com.ruoyi.business.domain.HwDeviceMode;
import com.ruoyi.business.domain.HwDeviceModeFunction;
import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.service.IHwDeviceModeService;
import com.ruoyi.business.service.IHwSceneService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 设备模型Controller
 *
 * @author xins
 * @date 2023-09-04
 */
@RestController
@RequestMapping("/deviceMode")
public class HwDeviceModeController extends BaseController {
    @Autowired
    private IHwDeviceModeService hwDeviceModeService;

    @Autowired
    private IHwSceneService hwSceneService;

    /**
     * 查询设备模型列表
     */
    @RequiresPermissions("business:deviceMode:list")
    @GetMapping("/list")
    public TableDataInfo list(HwDeviceMode hwDeviceMode) {
        startPage();
        hwDeviceMode.setDeviceModeStatus(HwDictConstants.DEVICE_MODE_STATUS_NORMAL);
        List<HwDeviceMode> list = hwDeviceModeService.selectHwDeviceModeJoinList(hwDeviceMode);
        return getDataTable(list);
    }

    /**
     * 导出设备模型列表
     */
    @RequiresPermissions("business:deviceMode:export")
    @Log(title = "设备模型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwDeviceMode hwDeviceMode) {
        List<HwDeviceMode> list = hwDeviceModeService.selectHwDeviceModeJoinList(hwDeviceMode);
        ExcelUtil<HwDeviceMode> util = new ExcelUtil<HwDeviceMode>(HwDeviceMode.class);
        util.exportExcel(response, list, "设备模型数据");
    }

    /**
     * 导出设备模型列表
     */
    @RequiresPermissions("business:deviceMode:export")
    @Log(title = "设备模型功能", businessType = BusinessType.EXPORT)
    @PostMapping("/exportFunction")
    public void exportFunction(HttpServletResponse response, HwDeviceMode hwDeviceMode) {
        List<HwDeviceModeFunction> functions = hwDeviceModeService.selectHwDeviceModeFunctionList(new HwDeviceModeFunction());
        ExcelUtil<HwDeviceModeFunction> functionUtil = new ExcelUtil<HwDeviceModeFunction>(HwDeviceModeFunction.class);
        functionUtil.exportExcel(response, functions, "设备模型功能数据");
    }

    /**
     * 获取设备模型详细信息
     */
    @RequiresPermissions("business:deviceMode:query")
    @GetMapping(value = "/{deviceModeId}")
    public AjaxResult getInfo(@PathVariable("deviceModeId") Long deviceModeId) {
        HwDeviceMode hwDeviceMode = hwDeviceModeService.selectHwDeviceModeByDeviceModeId(deviceModeId);
        List<HwDeviceModeFunction> hwDeviceModeFunctions = hwDeviceMode.getHwDeviceModeFunctionList();
        hwDeviceMode.setHwDeviceModeFunctionList(null);
        Map<String, List<HwDeviceModeFunction>> hwDeviceModeFunctionMap = hwDeviceModeFunctions.stream()
                .collect(Collectors.groupingBy(HwDeviceModeFunction::getFunctionMode));
        Map<String, Object> map = new HashMap<>();
        map.put("deviceMode", hwDeviceMode);
        map.put("deviceModeFunctionMap", hwDeviceModeFunctionMap);
        return success(map);
    }

    /**
     * 新增设备模型
     */
    @RequiresPermissions("business:deviceMode:add")
    @Log(title = "设备模型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwDeviceMode hwDeviceMode) {
        hwDeviceMode.setTenantId(SecurityUtils.getTenantId());
        hwDeviceMode.setCreateBy(SecurityUtils.getUsername());
        return toAjax(hwDeviceModeService.insertHwDeviceMode(hwDeviceMode));
    }

    /**
     * 修改设备模型
     */
    @RequiresPermissions("business:deviceMode:edit")
    @Log(title = "设备模型", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwDeviceMode hwDeviceMode) {
        return toAjax(hwDeviceModeService.updateHwDeviceMode(hwDeviceMode));
    }

    /**
     * 删除设备模型
     */
    @RequiresPermissions("business:deviceMode:remove")
    @Log(title = "设备模型", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deviceModeIds}")
    public AjaxResult remove(@PathVariable Long[] deviceModeIds) {
        return toAjax(hwDeviceModeService.deleteHwDeviceModeByDeviceModeIds(deviceModeIds));
    }

    @RequiresPermissions("business:deviceMode:list")
    @GetMapping("/getScenes")
    public AjaxResult getScenes(HwScene scene) {
        return success(hwSceneService.selectHwSceneList(scene));
    }

    @RequiresPermissions("business:deviceMode:list")
    @GetMapping("/getEditedScenes")
    public AjaxResult getEditedScenes(HwScene scene) {

        return success(hwSceneService.selectHwSceneList4Select(scene));
    }

    @RequiresPermissions("business:deviceMode:rebuild")
    @Log(title = "设备模型", businessType = BusinessType.REBUILD)
    @GetMapping("/rebuildTdSuperTables")
    public void rebuildTdSuperTables() {
        hwDeviceModeService.rebuildTdSuperTables();
    }
}
