package com.ruoyi.business.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.service.IHwSceneService;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.business.domain.HwElectronicFence;
import com.ruoyi.business.service.IHwElectronicFenceService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.page.TableDataInfo;

/**
 * 电子围栏Controller
 * 
 * @author xins
 * @date 2023-09-15
 */
@RestController
@RequestMapping("/electronicFence")
public class HwElectronicFenceController extends BaseController
{
    @Autowired
    private IHwElectronicFenceService hwElectronicFenceService;
    @Autowired
    private IHwSceneService hwSceneService;
    /**
     * 查询电子围栏列表
     */
    @RequiresPermissions("business:electronicFence:list")
    @GetMapping("/list")
    public TableDataInfo list(HwElectronicFence hwElectronicFence)
    {
        startPage();
        List<HwElectronicFence> list = hwElectronicFenceService.selectHwElectronicFenceJoinList(hwElectronicFence);
        return getDataTable(list);
    }

    /**
     * 导出电子围栏列表
     */
    @RequiresPermissions("business:electronicFence:export")
    @Log(title = "电子围栏", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwElectronicFence hwElectronicFence)
    {
        List<HwElectronicFence> list = hwElectronicFenceService.selectHwElectronicFenceList(hwElectronicFence);
        ExcelUtil<HwElectronicFence> util = new ExcelUtil<HwElectronicFence>(HwElectronicFence.class);
        util.exportExcel(response, list, "电子围栏数据");
    }

    /**
     * 获取电子围栏详细信息
     */
    @RequiresPermissions("business:electronicFence:query")
    @GetMapping(value = "/{electronicFenceId}")
    public AjaxResult getInfo(@PathVariable("electronicFenceId") Long electronicFenceId)
    {
        return success(hwElectronicFenceService.selectHwElectronicFenceByElectronicFenceId(electronicFenceId));
    }

    /**
     * 新增电子围栏
     */
    @RequiresPermissions("business:electronicFence:add")
    @Log(title = "电子围栏", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwElectronicFence hwElectronicFence)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getSysUser();
        hwElectronicFence.setTenantId(user.getTenantId());
        hwElectronicFence.setCreateBy(user.getUserName());
        return toAjax(hwElectronicFenceService.insertHwElectronicFence(hwElectronicFence));
    }

    /**
     * 修改电子围栏
     */
    @RequiresPermissions("business:electronicFence:edit")
    @Log(title = "电子围栏", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwElectronicFence hwElectronicFence)
    {
        return toAjax(hwElectronicFenceService.updateHwElectronicFence(hwElectronicFence));
    }

    /**
     * 删除电子围栏
     */
    @RequiresPermissions("business:electronicFence:remove")
    @Log(title = "电子围栏", businessType = BusinessType.DELETE)
	@DeleteMapping("/{electronicFenceIds}")
    public AjaxResult remove(@PathVariable Long[] electronicFenceIds)
    {
        return toAjax(hwElectronicFenceService.deleteHwElectronicFenceByElectronicFenceIds(electronicFenceIds));
    }

    @RequiresPermissions("business:electronicFence:list")
    @GetMapping("/getScenes")
    public AjaxResult getScenes(HwScene scene)
    {
        return success(hwSceneService.selectHwSceneList(scene));
    }

    @RequiresPermissions("business:electronicFence:list")
    @GetMapping("/getEditedScenes")
    public AjaxResult getEditedScenes(HwScene scene)
    {
        return success(hwSceneService.selectHwSceneList4Select(scene));
    }
}
