package com.ruoyi.business.controller;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.business.domain.*;
import com.ruoyi.business.domain.VO.*;
import com.ruoyi.business.service.*;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.tdengine.api.domain.TdReturnDataVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/monitorPlatform")
public class HwMonitorPlatformController extends BaseController {

    @Autowired
    private IHwMonitorUnitService hwMonitorUnitService;
    @Autowired
    private IHwDeviceService hwDeviceService;
    @Autowired
    private IHwDeviceModeService hwDeviceModeService;
    @Autowired
    private IHwSceneService hwSceneService;
    @Autowired
    private IHwElectronicFenceService hwElectronicFenceService;
    @Autowired
    private IHwAlarmInfoService hwAlarmInfoService;

    @Autowired
    private IHwMonitorPlatformService hwMonitorPlatformService;

    /**
     * 查询3.	设备各类型占所有设备数量百分比饼状图（首页左下角）
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/monitorUnitPercentage")
    public TableDataInfo monitorUnitPercentage() {
        List<DeviceModeVo> maps = hwDeviceService.selectDeviceModeNameVo();
        return getDataTable(maps);
    }

    /**
     * 各场景监控单元数量占监控单元总数量百分比图(首页左上)
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/monitorPercentage")
    public TableDataInfo monitorPercentage() {
        List<HwMonitorUnitVo> maps = hwMonitorUnitService.selectMonitorPercentage();
        return getDataTable(maps);
    }

    /**
     * 	展示总的设备数量和总的监控单元数量（首页中间上方）
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/allNums")
    public AllNumsVo allNums() {
        AllNumsVo allNumsVos = hwMonitorUnitService.selectAllNums(null);

        return allNumsVos;
    }

    /**
     * @return TableDataInfo
     * @param: hwAlarmInfo
     * @description 获取报警信息（首页右下角，定位场景首页，标准场景首页）
     * @author xins
     * @date 2023-09-15 11:03
     */
    @GetMapping("/getAlarmInfos")
    @RequiresPermissions("business:monitor:alarm")
    public TableDataInfo getAlarmInfos(HwAlarmInfo hwAlarmInfo) {
        //startPage();
        hwAlarmInfo.setHandleStatus(HwDictConstants.ALARM_HANDLE_STATUS_NO);
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoService.selectHwAlarmInfoList(hwAlarmInfo);
        return getDataTable(alarmInfos);
    }

    @PutMapping("/handleAlarmInfo")
    @Log(title = "监控平台处理报警信息", businessType = BusinessType.UPDATE)
    @RequiresPermissions("business:monitor:alarm")
    public AjaxResult handleAlarmInfo(@RequestBody HwAlarmInfo hwAlarmInfo) {
        hwAlarmInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(hwAlarmInfoService.updateHwAlarmInfo(hwAlarmInfo));
    }

    /**
     * 2.	设备实时在运数量曲线图(首页右上角)
     *
     * @param sceneId 场景id
     */
    @RequiresPermissions("business:monitor:index")
    @GetMapping("/getDeviceOperations/{sceneId}")
    public AjaxResult selectDeviceStatus(@PathVariable("sceneId") Long sceneId) {
        System.out.println(("sceneid:" + sceneId));
        return AjaxResult.success(hwDeviceService.getOnlineDevicesCount(sceneId));
    }


    /**
     * 	按场景展示总的设备数量和总的监控单元数量（标准场景）
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/sceneAllNums/{sceneId}")
    public AllNumsVo sceneAllNums(@PathVariable("sceneId") Long sceneId) {
        AllNumsVo allNumsVos = hwMonitorUnitService.selectAllNums(sceneId);
        return allNumsVos;
    }

    /**
     * 分页获取最子集监控单元列表(标准场景右侧)
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/limitSubMonitorUnit/{sceneId}")
    public TableDataInfo limitSubMonitorUnit(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit hwMonitorUnit = new HwMonitorUnit();
        hwMonitorUnit.setSceneId(sceneId);
        startPage();
        List<HwMonitorUnit> hwMonitorUnits = hwMonitorUnitService.selectLimitSubMonitorUnit(hwMonitorUnit);
        return getDataTable(hwMonitorUnits);
    }

    /**
     * @return AjaxResult
     * @param: sceneId
     * @description 标准场景按报警类型返回其报警数量
     * @author xins
     * @date 2023-09-21 11:35
     */
    @RequiresPermissions("business:monitor:standard")
    @GetMapping("/getAlarmCountGroupAlarmType/{sceneId}")
    public AjaxResult getAlarmCountGroupAlarmType(@PathVariable("sceneId") Long sceneId) {
        return success(hwAlarmInfoService.selectAlarmsByAlarmType(sceneId));
    }

    /**
     * 按电子围栏定位显示其下设备数量，以及报警的设备数量(定位场景地图气泡图)
     *
     * @param sceneId
     */
    @RequiresPermissions("business:monitor:gps")
    @GetMapping("/ElectronicNumVo/{sceneId}")
    public AjaxResult ElectronicNumVo(@PathVariable("sceneId") Long sceneId) {

        return success(hwElectronicFenceService.selectEleTypeNums(sceneId));
    }


    /**
     * 监控单元报警统计
     */
    @GetMapping("/alarmStats/{sceneId}")
    public AjaxResult alarmStats(@PathVariable("sceneId") Long sceneId) {
        List<AlarmStatsVo> alarmStatsVos = hwMonitorUnitService.selectAlarmStats(sceneId);
        return success(alarmStatsVos);
    }

    /**
     * 获取最子级监控单元数量、设备数量和在线设备数量（定位场景）
     *
     * @param sceneId 场景id
     */
    @RequiresPermissions("business:monitor:gps")
    @GetMapping("/subDevice/{sceneId}")
    public SubDeviceSumVo subDevice(@PathVariable("sceneId") Long sceneId) {
        return hwMonitorUnitService.selectSubDeviceSum(sceneId);
    }


    /**
     * 树状获取监控单元列表(设备监测左侧树)
     *
     * @param sceneId 场景ID
     * @return list
     * @throws
     */
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping("/treeList/{sceneId}")
    public AjaxResult monitorUnitTree(@PathVariable("sceneId") Long sceneId) {
        HwMonitorUnit queryMonitorUnit = new HwMonitorUnit();
        queryMonitorUnit.setSceneId(sceneId);
        return success(hwMonitorUnitService.selectMonitorTreeList(queryMonitorUnit));
    }

    /**
     * 获取监控单元详细信息
     */
//    @RequiresPermissions("business:monitorUnit:query")
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping(value = "/getMonitorById/{monitorUnitId}")
    public AjaxResult getMonitorById(@PathVariable("monitorUnitId") Long monitorUnitId) {
        return success(hwMonitorUnitService.selectHwMonitorUnitJoinByMonitorUnitId(monitorUnitId));
    }

    /**
     * 根据监控单元ID获取设备信息，分可控制设备和上传数据设备（设备监测页面）
     *
     * @param monitorUnitId 监控单元id
     */
    @RequiresPermissions("business:monitor:deviceMonitor")
    @GetMapping("/selectDevicesByMonitorUnitId/{monitorUnitId}/{sceneId}")
    public AjaxResult selectDeviceByDeviceModeByMonitorUnitId(@PathVariable("monitorUnitId") Long monitorUnitId,
                                                              @PathVariable("sceneId") Long sceneId) {
        HwDevice queryHwDevice = new HwDevice();
        queryHwDevice.setMonitorUnitId(monitorUnitId);
        queryHwDevice.setSceneId(sceneId);

        return success(hwDeviceService.getDevicesByMonitor(queryHwDevice));
    }


    @RequiresPermissions("business:monitor:deviceMonitor")
    @Log(title = "监控平台下发控制指令", businessType = BusinessType.CONTROL)
    @PostMapping("/publishControlCommand")
    public AjaxResult publishControlCommand(@RequestBody DeviceControlVo deviceControlVo) {
        hwDeviceService.publishControlCommand(deviceControlVo);
        return success();
    }


    /**
     * @description 根据场景ID获取设备模型列表(设备汇总页面)
     * @author xins
     * @date 2023-09-16 12:44
     */
    @RequiresPermissions("business:monitor:deviceList")
    @PostMapping("/getDeviceModesBySceneId")
    public AjaxResult getDeviceModesBySceneId(@RequestBody HwDeviceMode queryHwDeviceMode) {
        return success(hwDeviceModeService.selectHwDeviceModeList(queryHwDeviceMode));
    }

    /**
     * @return AjaxResult
     * @param: monitorUnitId
     * @description 根据设备模型获取最新数据（设备汇总页面）
     * @author xins
     * @date 2023-09-16 12:45
     */
    @RequiresPermissions("business:monitor:deviceList")
    @PostMapping("/selectLatestDataByDeviceMode")
    public AjaxResult selectLatestDataByDeviceMode(@Validated @RequestBody DeviceLatestDataVo deviceLatestData) {
        JSONObject returnObj = hwMonitorPlatformService.selectLatestDataByTags(deviceLatestData);
        return success(returnObj);
    }

    @RequiresPermissions("business:monitor:deviceInfo")
    @GetMapping("/getDeviceInfo/{deviceId}")
    public AjaxResult getDeviceInfo(@PathVariable("deviceId") Long deviceId) {
        HwDevice device = hwDeviceService.selectHwDeviceJoinByDeviceId(deviceId);
        return success(device);
    }

    @RequiresPermissions("business:monitor:deviceInfo")
    @GetMapping("/getLinkDevices/{deviceId}")
    public AjaxResult getLinkDevices(@PathVariable("deviceId") Long deviceId) {
        return success(hwDeviceService.selectLinkDeviceList(deviceId));
    }

    /**
     * @return AjaxResult
     * @param: deviceHistoryData
     * @description 获取设备历史数据
     * @author xins
     * @date 2023-09-16 15:15
     */
    @RequiresPermissions("business:monitor:deviceInfo")
    @PostMapping("/getHistoryData")
    public AjaxResult getHistoryData(@Validated @RequestBody DeviceHistoryDataVo deviceHistoryData) {
        JSONObject returnObj = new JSONObject();
        TdReturnDataVo returnDataVo = hwMonitorPlatformService.getHistoryData(deviceHistoryData);
        if (returnDataVo != null) {
            List<DeviceDataColumnVo> deviceDataColumnVos = hwMonitorPlatformService.
                    getDeviceDataColumns(deviceHistoryData.getDeviceModeId(), returnDataVo.getDataList());
            returnObj.put("deviceDataColumns", deviceDataColumnVos);
            returnObj.put("historyData", returnDataVo);
        }

        return success(returnObj);

    }


    /**
     * 获取场景，如果是管理员，则可以获取所有场景
     */
    @GetMapping("/selectSecnes")
    public AjaxResult selectSecnes(HwScene scene) {
        List<HwScene> hwScenes = hwSceneService.selectHwSceneList(scene);
        return success(hwScenes);
    }

}
