package com.ruoyi.business.mapper;

import java.util.List;
import com.ruoyi.business.domain.HwElectronicFence;
import com.ruoyi.business.domain.HwFenceArea;
import com.ruoyi.business.domain.VO.EleMonitorDeviceSumVo;


/**
 * 电子围栏Mapper接口
 *
 * @author xins
 * @date 2023-09-04
 */
public interface HwElectronicFenceMapper
{
    /**
     * 查询电子围栏
     *
     * @param electronicFenceId 电子围栏主键
     * @return 电子围栏
     */
    public HwElectronicFence selectHwElectronicFenceByElectronicFenceId(Long electronicFenceId);

    /**
     * 根据场景查询电子围栏信息
     * */
    public List<HwElectronicFence> selectElectronicByScene(Long sceneId);
    /**
     * 查询电子围栏列表
     *
     * @param hwElectronicFence 电子围栏
     * @return 电子围栏集合
     */
    public List<HwElectronicFence> selectHwElectronicFenceList(HwElectronicFence hwElectronicFence);

    /**
     * 新增电子围栏
     *
     * @param hwElectronicFence 电子围栏
     * @return 结果
     */
    public int insertHwElectronicFence(HwElectronicFence hwElectronicFence);
        //根据传入场景查询电子围栏的类型
    List<Integer> selectEleTypeBySceneId(Long sceneId);
        //查询监控单元下拥有总的设备量 根据监控单元分组
    List<EleMonitorDeviceSumVo> selectSumsFromMonitor(Long sceneId);
    //查询该场景下各个监控单元下拥有电子围栏设备的数量
    List<EleMonitorDeviceSumVo> selectSumsFromMonitorByElectronicFence(Long sceneId);
    //查询该场景下拥有电子围栏的设备的数量并按电子围栏分组
    List<EleMonitorDeviceSumVo> selectDevicesFromEleBySceneId(Long sceneId);
    //当电子围栏下只有监控单元时
    List<EleMonitorDeviceSumVo> selectEleOnlyMonitorBySceneId(Long sceneId);
    //当电子围栏下只有场景时
    Integer selectEleOnlyScene(Long sceneId);
    /**
     * 修改电子围栏
     *
     * @param hwElectronicFence 电子围栏
     * @return 结果
     */
    public int updateHwElectronicFence(HwElectronicFence hwElectronicFence);

    /**
     * 删除电子围栏
     *
     * @param electronicFenceId 电子围栏主键
     * @return 结果
     */
    public int deleteHwElectronicFenceByElectronicFenceId(Long electronicFenceId);

    /**
     * @param: electronicFenceIds
     * @description 删除fencearea
     * @author xins
     * @date 2023-09-27 16:10
     * @return int
     */
    public int deleteHwFenceAreaByElectronicFenceIds(Long[] electronicFenceIds);
    /**
     * 批量删除电子围栏
     *
     * @param electronicFenceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteHwElectronicFenceByElectronicFenceIds(Long[] electronicFenceIds);

    public int selectDeviceCount(Long electronicFenceId);

    public int selectDeviceCountByMonitorUnit(Long electronicFenceId);

    public int selectDeviceCountByScene(Long sceneId);

    /**
     * 查询电子围栏列表Join场景
     *
     * @param hwElectronicFence 电子围栏
     * @return 电子围栏集合
     */
    public List<HwElectronicFence> selectHwElectronicFenceJoinList(HwElectronicFence hwElectronicFence);

    /**
     * 批量新增电子围栏区域
     *
     * @param hwFenceAreaList 电子围栏区域列表
     * @return 结果
     */
    public int batchHwFenceArea(List<HwFenceArea> hwFenceAreaList);

    /**
     * 通过电子围栏主键删除电子围栏区域信息
     *
     * @param electronicFenceId 电子围栏ID
     * @return 结果
     */
    public int deleteHwFenceAreaByElectronicFenceId(Long electronicFenceId);
}
