package com.ruoyi.business.service.impl;

import java.util.List;

import com.ruoyi.business.domain.HwAlarmRule;
import com.ruoyi.business.mapper.HwAlarmRuleMapper;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.HwAlarmLevelMapper;
import com.ruoyi.business.domain.HwAlarmLevel;
import com.ruoyi.business.service.IHwAlarmLevelService;

/**
 * 报警级别Service业务层处理
 *
 * @author xins
 * @date 2023-09-15
 */
@Service
public class HwAlarmLevelServiceImpl implements IHwAlarmLevelService {
    @Autowired
    private HwAlarmLevelMapper hwAlarmLevelMapper;

    @Autowired
    private HwAlarmRuleMapper hwAlarmRuleMapper;

    /**
     * 查询报警级别
     *
     * @param alarmLevelId 报警级别主键
     * @return 报警级别
     */
    @Override
    public HwAlarmLevel selectHwAlarmLevelByAlarmLevelId(Long alarmLevelId) {
        return hwAlarmLevelMapper.selectHwAlarmLevelByAlarmLevelId(alarmLevelId);
    }

    /**
     * 查询报警级别列表
     *
     * @param hwAlarmLevel 报警级别
     * @return 报警级别
     */
    @Override
    @DataScope(tenantAlias = "hal")
    public List<HwAlarmLevel> selectHwAlarmLevelList(HwAlarmLevel hwAlarmLevel) {
        return hwAlarmLevelMapper.selectHwAlarmLevelList(hwAlarmLevel);
    }


    /**
     * 查询报警级别列表
     *
     * @param hwAlarmLevel 报警级别
     * @return 报警级别
     */
    @Override
    @DataScope(tenantAlias = "hal")
    public List<HwAlarmLevel> selectHwAlarmLevelJoinList(HwAlarmLevel hwAlarmLevel) {
        return hwAlarmLevelMapper.selectHwAlarmLevelJoinList(hwAlarmLevel);
    }

    /**
     * 新增报警级别
     *
     * @param hwAlarmLevel 报警级别
     * @return 结果
     */
    @Override
    public int insertHwAlarmLevel(HwAlarmLevel hwAlarmLevel) {
        hwAlarmLevel.setCreateTime(DateUtils.getNowDate());
        hwAlarmLevel.setTenantId(SecurityUtils.getTenantId());
        hwAlarmLevel.setCreateBy(SecurityUtils.getUsername());
        hwAlarmLevel.setCommonFlag(HwDictConstants.ALARM_LEVEL_COMMON_FLAG_NO);

        return hwAlarmLevelMapper.insertHwAlarmLevel(hwAlarmLevel);
    }

    /**
     * 修改报警级别
     *
     * @param hwAlarmLevel 报警级别
     * @return 结果
     */
    @Override
    public int updateHwAlarmLevel(HwAlarmLevel hwAlarmLevel) {
        hwAlarmLevel.setUpdateTime(DateUtils.getNowDate());
        hwAlarmLevel.setUpdateBy(SecurityUtils.getUsername());
        return hwAlarmLevelMapper.updateHwAlarmLevel(hwAlarmLevel);
    }

    /**
     * 批量删除报警级别
     *
     * @param alarmLevelIds 需要删除的报警级别主键
     * @return 结果
     */
    @Override
    public int deleteHwAlarmLevelByAlarmLevelIds(Long[] alarmLevelIds) {

        return hwAlarmLevelMapper.deleteHwAlarmLevelByAlarmLevelIds(alarmLevelIds);
    }

    /**
     * 删除报警级别信息
     *
     * @param alarmLevelId 报警级别主键
     * @return 结果
     */
    @Override
    public int deleteHwAlarmLevelByAlarmLevelId(Long alarmLevelId) {
        HwAlarmRule queryAlarmRule = new HwAlarmRule();
        queryAlarmRule.setAlarmLevelId(alarmLevelId);
        List<HwAlarmRule> alarmRuleList = hwAlarmRuleMapper.selectHwAlarmRuleList(queryAlarmRule);
        if (alarmRuleList != null && !alarmRuleList.isEmpty()) {
            throw new ServiceException("已有设备报警规则关联，不能删除!");
        }
        return hwAlarmLevelMapper.deleteHwAlarmLevelByAlarmLevelId(alarmLevelId);
    }
}
