package com.ruoyi.business.service.impl;

import com.ruoyi.business.domain.HwAlarmRule;
import com.ruoyi.business.domain.HwAlarmType;
import com.ruoyi.business.mapper.HwAlarmRuleMapper;
import com.ruoyi.business.mapper.HwAlarmTypeMapper;
import com.ruoyi.business.service.IHwAlarmTypeService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 报警类型Service业务层处理
 *
 * @author xins
 * @date 2023-09-15
 */
@Service
public class HwAlarmTypeServiceImpl implements IHwAlarmTypeService {
    @Autowired
    private HwAlarmTypeMapper hwAlarmTypeMapper;

    @Autowired
    private HwAlarmRuleMapper hwAlarmRuleMapper;

    /**
     * 查询报警类型
     *
     * @param alarmTypeId 报警类型主键
     * @return 报警类型
     */
    @Override
    public HwAlarmType selectHwAlarmTypeByAlarmTypeId(Long alarmTypeId) {
        return hwAlarmTypeMapper.selectHwAlarmTypeByAlarmTypeId(alarmTypeId);
    }

    /**
     * 查询报警类型列表
     *
     * @param hwAlarmType 报警类型
     * @return 报警类型
     */
    @Override
    @DataScope(tenantAlias = "hat")
    public List<HwAlarmType> selectHwAlarmTypeList(HwAlarmType hwAlarmType) {
        return hwAlarmTypeMapper.selectHwAlarmTypeList(hwAlarmType);
    }

    /**
     * 查询报警类型列表Join
     *
     * @param hwAlarmType 报警类型
     * @return 报警类型
     */
    @Override
    @DataScope(tenantAlias = "hat")
    public List<HwAlarmType> selectHwAlarmTypeJoinList(HwAlarmType hwAlarmType) {
        return hwAlarmTypeMapper.selectHwAlarmTypeJoinList(hwAlarmType);
    }

    /**
     * 新增报警类型
     *
     * @param hwAlarmType 报警类型
     * @return 结果
     */
    @Override
    public int insertHwAlarmType(HwAlarmType hwAlarmType) {
        hwAlarmType.setCreateTime(DateUtils.getNowDate());
        hwAlarmType.setCreateBy(SecurityUtils.getUsername());
        hwAlarmType.setTenantId(SecurityUtils.getTenantId());
        hwAlarmType.setCommonFlag(HwDictConstants.ALARM_TYPE_COMMON_FLAG_NO);

        return hwAlarmTypeMapper.insertHwAlarmType(hwAlarmType);
    }

    /**
     * 修改报警类型
     *
     * @param hwAlarmType 报警类型
     * @return 结果
     */
    @Override
    public int updateHwAlarmType(HwAlarmType hwAlarmType) {
        hwAlarmType.setUpdateTime(DateUtils.getNowDate());
        hwAlarmType.setUpdateBy(SecurityUtils.getUsername());
        return hwAlarmTypeMapper.updateHwAlarmType(hwAlarmType);
    }

    /**
     * 批量删除报警类型
     *
     * @param alarmTypeIds 需要删除的报警类型主键
     * @return 结果
     */
    @Override
    public int deleteHwAlarmTypeByAlarmTypeIds(Long[] alarmTypeIds) {
        return hwAlarmTypeMapper.deleteHwAlarmTypeByAlarmTypeIds(alarmTypeIds);
    }

    /**
     * 删除报警类型信息
     *
     * @param alarmTypeId 报警类型主键
     * @return 结果
     */
    @Override
    public int deleteHwAlarmTypeByAlarmTypeId(Long alarmTypeId) {
        HwAlarmRule queryAlarmRule = new HwAlarmRule();
        queryAlarmRule.setAlarmTypeId(alarmTypeId);
        List<HwAlarmRule> alarmRuleList = hwAlarmRuleMapper.selectHwAlarmRuleList(queryAlarmRule);
        if (alarmRuleList != null && !alarmRuleList.isEmpty()) {
            throw new ServiceException("已有设备报警规则关联，不能删除!");
        }
        return hwAlarmTypeMapper.deleteHwAlarmTypeByAlarmTypeId(alarmTypeId);
    }
}
