package com.ruoyi.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.business.domain.HwAlarmInfo;
import com.ruoyi.business.domain.HwElectronicFence;
import com.ruoyi.business.domain.HwFenceArea;
import com.ruoyi.business.domain.VO.AlarmInfoVo;
import com.ruoyi.business.domain.VO.ElectronicFenceVo;
import com.ruoyi.business.mapper.HwAlarmInfoMapper;
import com.ruoyi.business.mapper.HwElectronicFenceMapper;
import com.ruoyi.business.mapper.HwFenceAreaMapper;
import com.ruoyi.business.mapper.HwFenceTargetMapper;
import com.ruoyi.business.service.IHwElectronicFenceService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 电子围栏Service业务层处理
 *
 * @author xins
 * @date 2023-09-04
 */
@Service
public class HwElectronicFenceServiceImpl implements IHwElectronicFenceService {
    @Autowired
    private HwElectronicFenceMapper hwElectronicFenceMapper;

    @Autowired
    private HwFenceTargetMapper hwFenceTargetMapper;

    @Autowired
    private HwFenceAreaMapper hwFenceAreaMapper;

    @Autowired
    private HwAlarmInfoMapper hwAlarmInfoMapper;

    public static final String SEPARATOR_UNDERLINE = "_";
    public static final String SEPARATOR_COMMA = ",";
    public static final String KEY_FENCE_AREA_ID = "fenceAreaId";
    public static final String KEY_COORDINATES = "coordinates";
    public static final String KEY_LONGITUDE = "longitude";
    public static final String KEY_LATITUDE = "latitude";
    public static final String KEY_CENTER = "center";
    public static final String KEY_RADIUS = "radius";

    /**
     * 查询电子围栏
     *
     * @param electronicFenceId 电子围栏主键
     * @return 电子围栏
     */
    @Override
    public HwElectronicFence selectHwElectronicFenceByElectronicFenceId(Long electronicFenceId) {
        HwElectronicFence electronicFence = hwElectronicFenceMapper.selectHwElectronicFenceByElectronicFenceId(electronicFenceId);
        HwFenceArea queryFenceArea = new HwFenceArea();
        queryFenceArea.setElectronicFenceId(electronicFence.getElectronicFenceId());
        List<HwFenceArea> fenceAreas = hwFenceAreaMapper.selectHwFenceAreaList(queryFenceArea);
        JSONArray polygonJsonListArr = new JSONArray();
        JSONArray circleJsonListArr = new JSONArray();
        if (fenceAreas != null && !fenceAreas.isEmpty()) {
            int polygonIndex = 0;
            int circleIndex = 0;
            for (HwFenceArea fenceArea : fenceAreas) {
                String areaShapeFlag = fenceArea.getAreaShapeFlag();
                String areaRange = fenceArea.getAreaRange();
                if (areaShapeFlag.equals(HwDictConstants.AREA_SHAPE_FLAG_POLYGN)) {
                    String[] polygonRangePointsArr = areaRange.split("_");
                    JSONArray polygonJsonArr = new JSONArray();
                    JSONObject polyJsonObject = new JSONObject();
                    for (String polygonRangePoint : polygonRangePointsArr) {
                        JSONObject polygonJson = new JSONObject();
                        polygonJson.put("longitude", new BigDecimal(polygonRangePoint.split(",")[0]));
                        polygonJson.put("latitude", new BigDecimal(polygonRangePoint.split(",")[1]));
                        polygonJson.put("index", polygonIndex);
                        polygonJsonArr.add(polygonJson);
                    }
                    polyJsonObject.put("fenceAreaId", fenceArea.getFenceAreaId());
                    polyJsonObject.put("polyJsonArr", polygonJsonArr.toString());
                    polygonJsonListArr.add(polyJsonObject.toJSONString());
                    polygonIndex++;

                } else if (areaShapeFlag.equals(HwDictConstants.AREA_SHAPE_FLAG_CIRCULA)) {
                    String[] circleRangePointsArr = areaRange.split(",");

                    JSONObject circleJson = new JSONObject();
                    JSONArray centerPointJsonArr = new JSONArray();
                    centerPointJsonArr.add(new BigDecimal(circleRangePointsArr[0]));
                    centerPointJsonArr.add(new BigDecimal(circleRangePointsArr[1]));
                    circleJson.put("center", centerPointJsonArr.toString());
                    circleJson.put("radius", new BigDecimal(circleRangePointsArr[2]));
                    circleJson.put("index", circleIndex);
                    circleJson.put("fenceAreaId", fenceArea.getFenceAreaId());
                    circleJsonListArr.add(circleJson);

                    circleIndex++;

                }
            }
        }
        electronicFence.setCircleAreaListStr(circleJsonListArr.toString());
        electronicFence.setPolygonAreaListStr(polygonJsonListArr.toString());

        HwAlarmInfo queryAlarmInfo = new HwAlarmInfo();
        queryAlarmInfo.setAlarmInfoType(HwDictConstants.ALARM_INFO_TYPE_ELECTRONIC_FENCE);
        queryAlarmInfo.setAlarmReleatedId(electronicFenceId);
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoMapper.selectHwAlarmInfoList(queryAlarmInfo);
        if (alarmInfos == null || alarmInfos.isEmpty()) {
            electronicFence.setAlarmInfoFlag("0");
        } else {
            electronicFence.setAlarmInfoFlag("1");
        }

        return electronicFence;
    }

    @Override
    public List<HwElectronicFence> selectElectronicByScene(Long sceneId) {
        return hwElectronicFenceMapper.selectElectronicByScene(sceneId);
    }

    @Override
    public List<ElectronicFenceVo> selectEleTypeNums(Long sceneId) {
        List<ElectronicFenceVo> electronicFenceVos = new ArrayList<>();
        HwElectronicFence queryElectronicFence = new HwElectronicFence();
        queryElectronicFence.setSceneId(sceneId);
        List<HwElectronicFence> electronicFenceList = hwElectronicFenceMapper.selectHwElectronicFenceList(queryElectronicFence);
        electronicFenceList.forEach(electronicFence -> {
            HwFenceArea queryFenceArea = new HwFenceArea();
            queryFenceArea.setElectronicFenceId(electronicFence.getElectronicFenceId());
            List<HwFenceArea> fenceAreas = hwFenceAreaMapper.selectHwFenceAreaList(queryFenceArea);
            for (HwFenceArea fenceArea : fenceAreas) {
                ElectronicFenceVo e = new ElectronicFenceVo();
                String fenceType = electronicFence.getFenceType();
                String areaShapeFlag = fenceArea.getAreaShapeFlag();
                String areaRange = fenceArea.getAreaRange();
                Integer deviceCount = 0;
                if (fenceType.equals(HwDictConstants.FENCE_TYPE_DEVICE)
                        || fenceType.equals(HwDictConstants.FENCE_TYPE_MONITOR_UNIT)) {
                    deviceCount = hwElectronicFenceMapper.selectDeviceCount(electronicFence.getElectronicFenceId());
                }

                if (fenceType.equals(HwDictConstants.FENCE_TYPE_MONITOR_UNIT)) {
                    deviceCount = hwElectronicFenceMapper.selectDeviceCountByMonitorUnit(electronicFence.getElectronicFenceId());

                } else if (fenceType.equals(HwDictConstants.FENCE_TYPE_SCENE)) {
                    deviceCount = hwElectronicFenceMapper.selectDeviceCountByScene(electronicFence.getSceneId());
                }

                if (areaShapeFlag.equals(HwDictConstants.AREA_SHAPE_FLAG_POLYGN)) {
                    String[] areaRangeArr = areaRange.split("_");
                    List<ElectronicFenceVo.AreaPoint> points = new ArrayList<ElectronicFenceVo.AreaPoint>();
                    for (String areaR : areaRangeArr) {
                        String[] areaRArr = areaR.split(",");
                        ElectronicFenceVo.AreaPoint p = new ElectronicFenceVo.AreaPoint();
                        p.setLongitude(Double.valueOf(areaRArr[0]));
                        p.setLatitude(Double.valueOf(areaRArr[1]));
                        points.add(p);
                    }
                    e.setAreaPoints(points);

                } else if (areaShapeFlag.equals(HwDictConstants.AREA_SHAPE_FLAG_CIRCULA)) {
                    String[] areaRangeArr = areaRange.split(",");
                    ElectronicFenceVo.AreaPoint p = new ElectronicFenceVo.AreaPoint();
                    p.setLongitude(Double.valueOf(areaRangeArr[0]));
                    p.setLatitude(Double.valueOf(areaRangeArr[1]));

                    e.setCenterPoint(p);
                    e.setRadius(Double.valueOf(areaRangeArr[2]));
                }

                AlarmInfoVo alarmInfoVo = hwAlarmInfoMapper.selectAlarmCountByFenceArea(fenceArea.getFenceAreaId());
                e.setAbnormalCount(alarmInfoVo.getAlarmCount());
                e.setAreaShapeFlag(areaShapeFlag);
                e.setDeviceCount(deviceCount);
                electronicFenceVos.add(e);
            }
        });

        return electronicFenceVos;
    }

    /**
     * 查询电子围栏列表
     *
     * @param hwElectronicFence 电子围栏
     * @return 电子围栏
     */
    @Override
    public List<HwElectronicFence> selectHwElectronicFenceList(HwElectronicFence hwElectronicFence) {
        return hwElectronicFenceMapper.selectHwElectronicFenceList(hwElectronicFence);
    }

    /**
     * 新增电子围栏
     *
     * @param hwElectronicFence 电子围栏
     * @return 结果
     */
    @Override
    @Transactional
    public int insertHwElectronicFence(HwElectronicFence hwElectronicFence) {
        hwElectronicFence.setFencePushFlag(HwDictConstants.FENCE_PUSH_FLAG_NO);
        hwElectronicFence.setFenceType(HwDictConstants.FENCE_TYPE_SCENE);
        hwElectronicFence.setCreateTime(DateUtils.getNowDate());
        int rows = hwElectronicFenceMapper.insertHwElectronicFence(hwElectronicFence);
        this.batchInsertFenceArea(hwElectronicFence);
        return rows;
    }

    private void batchInsertFenceArea(HwElectronicFence hwElectronicFence) {
        Long electronicFenceId = hwElectronicFence.getElectronicFenceId();
        String polygonAreaListStr = hwElectronicFence.getPolygonAreaListStr();
        String circleAreaListStr = hwElectronicFence.getCircleAreaListStr();
        List<HwFenceArea> fenceAreaList = new ArrayList<>();//所有需要插入的fencearea数据（包含有fenceareaid的）
        List<HwFenceArea> toInsertedFenceAreaList = new ArrayList<>();//只有需要insert的数据（没有fenceareaid待插入的数据）
        if (polygonAreaListStr != null) {//多个多边形
            JSONArray polygonAreasJsonArray = JSON.parseArray(polygonAreaListStr);
            polygonAreasJsonArray.forEach(polygonAreaJsonStr -> {//遍历多个多边形
                JSONObject polygonAreaJsonObj = (JSONObject) polygonAreaJsonStr;
                Object fenceAreaId = polygonAreaJsonObj.get(KEY_FENCE_AREA_ID);
                StringBuilder areaRange = new StringBuilder();

                polygonAreaJsonObj.getJSONArray(KEY_COORDINATES).forEach(polyJsonPointStr -> {//遍历每个多边形的多个点
                    JSONObject polyJsonPointJson = (JSONObject) polyJsonPointStr;
                    areaRange.append(SEPARATOR_UNDERLINE);
                    areaRange.append(polyJsonPointJson.getString(KEY_LONGITUDE)).append(SEPARATOR_COMMA)
                            .append(polyJsonPointJson.getString(KEY_LATITUDE));

                });

                HwFenceArea polygonArea = new HwFenceArea();
                polygonArea.setElectronicFenceId(electronicFenceId);
                polygonArea.setAreaShapeFlag(HwDictConstants.AREA_SHAPE_FLAG_POLYGN);
                polygonArea.setAreaRange(areaRange.toString().replaceFirst(SEPARATOR_UNDERLINE, ""));
                if (fenceAreaId != null && !"".equals(fenceAreaId)) {
                    polygonArea.setFenceAreaId(Long.valueOf(String.valueOf(fenceAreaId)));
                } else {
                    toInsertedFenceAreaList.add(polygonArea);
                }
                fenceAreaList.add(polygonArea);

            });

        }
        if (circleAreaListStr != null) {
            JSONArray circleAreaJsonArray = JSON.parseArray(circleAreaListStr);
            for (Object circleAreaObj : circleAreaJsonArray) {
                StringBuilder areaRange = new StringBuilder();
                JSONObject circleAreaJson = (JSONObject) circleAreaObj;
                JSONArray centerPointArr = circleAreaJson.getJSONArray(KEY_CENTER);
                String radius = circleAreaJson.getString(KEY_RADIUS);
                Object fenceAreaId = circleAreaJson.get(KEY_FENCE_AREA_ID);
                BigDecimal longitude = (BigDecimal) centerPointArr.get(0);
                BigDecimal latitude = (BigDecimal) centerPointArr.get(1);
                areaRange.append(longitude).append(SEPARATOR_COMMA)
                        .append(latitude).append(SEPARATOR_COMMA)
                        .append(radius);
                HwFenceArea polygonArea = new HwFenceArea();
                polygonArea.setElectronicFenceId(electronicFenceId);
                polygonArea.setAreaShapeFlag(HwDictConstants.AREA_SHAPE_FLAG_CIRCULA);
                polygonArea.setAreaRange(areaRange.toString());
                if (fenceAreaId != null && !"".equals(fenceAreaId)) {
                    polygonArea.setFenceAreaId(Long.valueOf(String.valueOf(fenceAreaId)));
                } else {
                    toInsertedFenceAreaList.add(polygonArea);
                }
                fenceAreaList.add(polygonArea);
            }
        }
        if (hwElectronicFence.getAlarmInfoFlag() == null ||
                HwDictConstants.ALARM_INFO_FLAG_NO.equals(hwElectronicFence.getAlarmInfoFlag())) {
            //如果没有关联报警信息，则全量插入
            hwElectronicFenceMapper.deleteHwFenceAreaByElectronicFenceId(hwElectronicFence.getElectronicFenceId());
            if (!fenceAreaList.isEmpty()) {
                hwElectronicFenceMapper.batchHwFenceArea(fenceAreaList);
            }
        } else {//如果有报警信息，则只插入新增的fencearea
            if (!toInsertedFenceAreaList.isEmpty()) {
                hwElectronicFenceMapper.batchHwFenceArea(toInsertedFenceAreaList);
            }
        }
    }

    /**
     * 修改电子围栏
     *
     * @param hwElectronicFence 电子围栏
     * @return 结果
     */
    @Override
    public int updateHwElectronicFence(HwElectronicFence hwElectronicFence) {
        HwAlarmInfo queryAlarmInfo = new HwAlarmInfo();
        queryAlarmInfo.setAlarmInfoType(HwDictConstants.ALARM_INFO_TYPE_ELECTRONIC_FENCE);
        queryAlarmInfo.setAlarmReleatedId(hwElectronicFence.getElectronicFenceId());
        List<HwAlarmInfo> alarmInfos = hwAlarmInfoMapper.selectHwAlarmInfoList(queryAlarmInfo);
        hwElectronicFence.setAlarmInfoFlag(HwDictConstants.ALARM_INFO_FLAG_YES);
        if (alarmInfos == null || alarmInfos.isEmpty()) {
            hwElectronicFence.setAlarmInfoFlag(HwDictConstants.ALARM_INFO_FLAG_NO);
        }

        this.batchInsertFenceArea(hwElectronicFence);
        hwElectronicFence.setUpdateTime(DateUtils.getNowDate());
        hwElectronicFence.setUpdateBy(SecurityUtils.getUsername());
        return hwElectronicFenceMapper.updateHwElectronicFence(hwElectronicFence);
    }

    /**
     * 批量删除电子围栏
     *
     * @param electronicFenceIds 需要删除的电子围栏主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteHwElectronicFenceByElectronicFenceIds(Long[] electronicFenceIds) {
        //逻辑删除，修改触发状态为9
        int rows = hwElectronicFenceMapper.deleteHwElectronicFenceByElectronicFenceIds(electronicFenceIds);
        // hwElectronicFenceMapper.deleteHwFenceAreaByElectronicFenceIds(electronicFenceIds);
        return rows;
    }

    /**
     * 删除电子围栏信息
     *
     * @param electronicFenceId 电子围栏主键
     * @return 结果
     */
    @Override
    public int deleteHwElectronicFenceByElectronicFenceId(Long electronicFenceId) {
        return hwElectronicFenceMapper.deleteHwElectronicFenceByElectronicFenceId(electronicFenceId);
    }


    /**
     * 查询电子围栏列表Join场景
     *
     * @param hwElectronicFence 电子围栏
     * @return 电子围栏
     */
    @Override
    @DataScope(tenantAlias = "hef")
    public List<HwElectronicFence> selectHwElectronicFenceJoinList(HwElectronicFence hwElectronicFence) {
        return hwElectronicFenceMapper.selectHwElectronicFenceJoinList(hwElectronicFence);
    }

}
