package com.ruoyi.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.business.domain.HwDevice;
import com.ruoyi.business.domain.HwScene;
import com.ruoyi.business.domain.VO.HwSceneVo;
import com.ruoyi.business.mapper.HwDeviceMapper;
import com.ruoyi.business.mapper.HwSceneMapper;
import com.ruoyi.business.service.IHwSceneService;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.tdengine.api.RemoteTdEngineService;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 场景信息Service业务层处理
 *
 * @author xins
 * @date 2023-08-23
 */
@Service
public class HwSceneServiceImpl implements IHwSceneService {
    @Autowired
    private HwSceneMapper hwSceneMapper;
    @Autowired
    private HwDeviceMapper hwDeviceMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;


    /**
     * 查询场景信息
     *
     * @param sceneId 场景信息主键
     * @return 场景信息
     */
    @Override
    public HwScene selectHwSceneBySceneId(Long sceneId) {
        return hwSceneMapper.selectHwSceneBySceneId(sceneId);
    }


    /**
     * 查询场景信息列表
     *
     * @param hwScene 场景信息
     * @return 场景信息
     */
    @Override
    @DataScope(tenantAlias = "hs")
    public List<HwScene> selectHwSceneList(HwScene hwScene) {
        hwScene.setSceneStatus(HwDictConstants.SCENE_STATUS_NORMAL);
        List<HwScene> scenes = hwSceneMapper.selectHwSceneList(hwScene);
        boolean defaultFlag = false;
        for (HwScene scene : scenes) {
            if (defaultFlag) {
                scene.setDefaultFlag(HwDictConstants.SCENE_DEFAULT_FLAG_NO);
            } else {
                if (scene.getDefaultFlag().equalsIgnoreCase(HwDictConstants.SCENE_DEFAULT_FLAG_YES)) {
                    defaultFlag = true;
                }
            }
            scene.setRouter(HwDictConstants.SCENE_MODE_ROUTER_MAP.get(scene.getSceneModeId()));
        }
        return scenes;
    }

    @Override
    public List<HwSceneVo> selectHwSceneJoinList(HwSceneVo hwScene) {
        hwScene.setSceneStatus(HwDictConstants.SCENE_STATUS_NORMAL);
        return hwSceneMapper.selectHwSceneVoList(hwScene);
    }

    /**
     * 新增场景信息
     *
     * @param hwScene 场景信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertHwScene(HwScene hwScene) {
        hwScene.setCreateBy(SecurityUtils.getUsername());

        if (hwScene.getPreserveTime() == null) {
            hwScene.setPreserveTime(new BigDecimal(90));
        }
        if (hwScene.getTestPreserveTime() == null) {
            hwScene.setTestPreserveTime(new BigDecimal(30));
        }

        String randomAccount = RandomStringUtils.random(6, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789");
        String randomKey = RandomStringUtils.random(12, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789");
        hwScene.setModeAccount(randomAccount);
        hwScene.setModeKey(randomKey);

        hwScene.setCreateTime(DateUtils.getNowDate());

        int rows = hwSceneMapper.insertHwScene(hwScene);
        this.updateDefaultFlag(hwScene);
//        this.createTdDatabase(hwScene);
        return rows;
    }

//    private void createTdDatabase(HwScene hwScene) {
//        String databaseName = TdEngineConstants.getDatabaseName(hwScene.getSceneId());
//        R<?> tdReturnMsg = this.remoteTdEngineService.createDataBase(databaseName);
//        if (tdReturnMsg.getCode() != Constants.SUCCESS) {//抛出异常，回滚事务
//            throw new RuntimeException(tdReturnMsg.getMsg());
//        }
//    }

    /**
     * 修改场景信息
     *
     * @param hwScene 场景信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateHwScene(HwScene hwScene) {
        this.updateDefaultFlag(hwScene);

        hwScene.setUpdateTime(DateUtils.getNowDate());
        int rows = hwSceneMapper.updateHwScene(hwScene);
        this.updateRedisAuth(hwScene);
        return rows;
    }

    /**
     * 更新redis的auth认证信息
     */
    private void updateRedisAuth(HwScene hwScene) {
        try {
            HwScene dbScene = hwSceneMapper.selectHwSceneBySceneId(hwScene.getSceneId());
            if (dbScene.getModeAccount().equals(hwScene.getModeAccount()) ||
                    dbScene.getModeKey().equals(hwScene.getModeKey())) {
                String modeAccount = hwScene.getModeAccount();
                String modeKey = hwScene.getModeKey();
                HwDevice queryDevice = new HwDevice();
                queryDevice.setSceneId(hwScene.getSceneId());
                List<HwDevice> devices = hwDeviceMapper.selectHwDeviceList(queryDevice);
                if (devices != null && !devices.isEmpty()) {
                    String deviceInfoStr = redisTemplate.opsForValue().get(HwDictConstants.REDIS_KEY_DEVICE_INFO);
                    JSONArray deviceInfoJsonArr = new JSONArray();
                    if (StringUtils.isNotEmpty(deviceInfoStr)) {
                        deviceInfoJsonArr = JSON.parseArray(deviceInfoStr);
                    }
                    for (HwDevice device : devices) {
                        boolean redisUpdated = false;//是否更新已有的值
                        for (Object deviceInfoObj : deviceInfoJsonArr) {
                            JSONObject deviceInfoJson = (JSONObject) deviceInfoObj;
                            if (deviceInfoJson.getString("deviceCode").equals(device.getDeviceCode())) {
                                deviceInfoJson.put("userName", modeAccount);
                                deviceInfoJson.put("password", modeKey);
                                redisUpdated = true;
                            }
                        }

                        if (!redisUpdated) {
                            JSONObject deviceInfoJson = new JSONObject();
                            deviceInfoJson.put("deviceCode", device.getDeviceCode());
                            deviceInfoJson.put("userName", modeAccount);
                            deviceInfoJson.put("password", modeKey);
                            deviceInfoJsonArr.add(deviceInfoJson);
                        }

                    }
                    redisTemplate.opsForValue().set(HwDictConstants.REDIS_KEY_DEVICE_INFO, deviceInfoJsonArr.toString());
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }
    }


    /**
     * 批量删除场景信息
     *
     * @param sceneIds 需要删除的场景信息主键
     * @return 结果
     */
    @Override
    public int deleteHwSceneBySceneIds(Long[] sceneIds) {
        for (Long sceneId : sceneIds) {
            HwScene scene = hwSceneMapper.selectHwSceneBySceneId(sceneId);
            scene.setSceneStatus(HwDictConstants.SCENE_STATUS_DELETED);
            hwSceneMapper.updateHwScene(scene);
            //TODO:后面可以增加更新redis的auth信息，并且删除tdengine的database
        }
        return 1;
    }

    /**
     * 删除场景信息信息
     *
     * @param sceneId 场景信息主键
     * @return 结果
     */
    @Override
    public int deleteHwSceneBySceneId(Long sceneId) {
        return hwSceneMapper.deleteHwSceneBySceneId(sceneId);
    }

    /**
     * 查询场景信息列表供其他页面选择使用
     *
     * @param hwScene 场景信息
     * @return 场景信息
     */
    @Override
    public List<HwScene> selectHwSceneList4Select(HwScene hwScene) {
        Long tenantId = SecurityUtils.getTenantId();
        //如果不是管理租户，需要过滤
        if(!tenantId.equals(HwDictConstants.ADMINISTRATOR_TENANT_ID)){
            hwScene.setTenantId(tenantId);
        }

        List<HwScene> scenes = hwSceneMapper.selectHwSceneList(hwScene);
        //如果是管理员租户，则需要过滤不是自己的只能显示，不能选择
        if(tenantId.equals(HwDictConstants.ADMINISTRATOR_TENANT_ID)) {
            scenes.forEach(s -> {
                if (!s.getTenantId().equals(tenantId)) {
                    s.setSelectedDisable("1");
                }
            });
        }
        return scenes;
    }

    /**
     * @param: hwScene
     * @description 更新场景默认标识，一个租户只能有一个默认场景
     * @author xins
     * @date 2023-09-26 18:19
     */
    private void updateDefaultFlag(HwScene hwScene) {
        if (hwScene.getDefaultFlag().equals(HwDictConstants.SCENE_DEFAULT_FLAG_YES)) {
            HwScene queryScene = new HwScene();
            queryScene.setTenantId(hwScene.getTenantId());
            List<HwScene> dbScenes = hwSceneMapper.selectHwSceneList(queryScene);
            if (dbScenes != null && !dbScenes.isEmpty()) {
                dbScenes.forEach(dbScene -> {
                    if (hwScene.getSceneId() == null || (hwScene.getSceneId() != null &&
                            !hwScene.getSceneId().equals(dbScene.getSceneId()))) {
                        dbScene.setDefaultFlag(HwDictConstants.SCENE_DEFAULT_FLAG_NO);
                        hwSceneMapper.updateHwScene(dbScene);
                    }
                });
            }
        }
    }

}
