package com.ruoyi.dataprocess.service;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.constant.HwDictConstants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

@Service
public class CommanHandleService {
    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * @param: controlCommandTopic 下发数据的主题
     * @param: subDeviceCode 子设备或直连设备设备编号
     * @param: serviceFunctionIdentifier
     * @description 联动设备发送指令
     * @author xins
     * @date 2023-11-07 16:28
     */
    public void publishControlCommand(String controlCommandTopic, String subDeviceCode, String serviceFunctionIdentifier) {
        JSONObject controlCommandJson = new JSONObject();
        JSONObject payloadJson = new JSONObject();
        JSONObject paramJson = new JSONObject();
        paramJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_UID_KEY, subDeviceCode);//子设备或直连设备设备编号

        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_MID_KEY, "");
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_TIMESTAMP_KEY, System.currentTimeMillis() / 1000);//单位：s
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_TYPE_KEY, serviceFunctionIdentifier);
        payloadJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_PARAM_KEY, paramJson);

        controlCommandJson.put(HwDictConstants.CONTROL_COMMAND_TOPIC_KEY, controlCommandTopic);
        controlCommandJson.put(HwDictConstants.CONTROL_COMMAND_PAYLOAD_KEY, payloadJson.toString());

        redisTemplate.convertAndSend(HwDictConstants.CONTROL_COMMAND_REDIS_KEY, controlCommandJson.toString());
    }
}
