package com.ruoyi.dataprocess.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.constant.SecurityConstants;
import com.ruoyi.common.core.constant.TdEngineConstants;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.dataprocess.domain.*;
import com.ruoyi.dataprocess.mapper.HwAlarmInfoMapper;
import com.ruoyi.dataprocess.mapper.HwDeviceMapper;
import com.ruoyi.dataprocess.mapper.HwOfflineRuleMapper;
import com.ruoyi.dataprocess.service.CommanHandleService;
import com.ruoyi.dataprocess.service.IDeviceStatusService;
import com.ruoyi.tdengine.api.RemoteTdEngineService;
import com.ruoyi.tdengine.api.domain.DeviceStatus;
import com.ruoyi.tdengine.api.domain.TdField;
import com.ruoyi.tdengine.api.domain.TdTableVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: 设备状态处理服务
 * @ClassName: DeviceStatusServiceImpl
 * @Author : xins
 * @Date :2023-09-12  15:31
 * @Version :1.0
 */
@Service
public class DeviceStatusServiceImpl extends CommanHandleService implements IDeviceStatusService {

    @Autowired
    private HwDeviceMapper hwDeviceMapper;
    @Autowired
    private HwOfflineRuleMapper hwOfflineRuleMapper;
    @Autowired
    private HwAlarmInfoMapper hwAlarmInfoMapper;
    @Resource
    private RemoteTdEngineService remoteTdEngineService;

    @Override
    public void handleDeviceStatus(String payloadString, String clientId) {
//        ddd:{"msg":"设备设备连接状态信息","deviceType":"edge","connectStatus":1,
//                "statusTime":1694506127199,"deviceCode":"hw-data-process-1"}
        JSONObject json = JSON.parseObject(payloadString);
        String deviceCode = json.getString("deviceCode").toLowerCase();
        if (clientId.equals(deviceCode)) { //校验是不是自己，如果是自己则不记录状态，返回即可。
            return;
        }
        HwDevice hwDevice = hwDeviceMapper.selectHwDeviceByDeviceCode(deviceCode);
        if (hwDevice != null) {
            Long deviceId = hwDevice.getDeviceId();
            TdTableVo tdTableVo = new TdTableVo();
            tdTableVo.setDatabaseName(TdEngineConstants.PLATFORM_DB_NAME);
            tdTableVo.setTableName(TdEngineConstants.DEFAULT_DEVICE_STATUS_TABLE_NAME_PREFIX + deviceId);

            List<TdField> schemaFields = new ArrayList<>();
            TdField onlineStatusField = new TdField();
            onlineStatusField.setFieldName(TdEngineConstants.ST_TAG_ONLINESTATUS);
            onlineStatusField.setFieldValue(json.getInteger("connectStatus"));
            schemaFields.add(onlineStatusField);

            TdField deviceTypeField = new TdField();
            deviceTypeField.setFieldName(TdEngineConstants.ST_TAG_DEVICETYPE);
            deviceTypeField.setFieldValue(json.getString("deviceType"));
            schemaFields.add(deviceTypeField);

            TdField tsField = new TdField();
            tsField.setFieldName(TdEngineConstants.DEFAULT_FIRST_FIELD_NAME);
            tsField.setFieldValue(json.getLong("statusTime"));
            schemaFields.add(tsField);

            tdTableVo.setSchemaFields(schemaFields);

            this.remoteTdEngineService.insertTable(tdTableVo, SecurityConstants.INNER);

            //更新设备当前状态信息
            String connectStatus = String.valueOf(json.getInteger("connectStatus"));
            hwDevice.setOnlineStatus(connectStatus);
            //判断如果是否是第一次连接，更新激活状态和激活时间。
            if (hwDevice.getActiveStatus().equals(HwDictConstants.DEVICE_ACTIVE_STATUS_INACTIVE)) {
                hwDevice.setActiveStatus(HwDictConstants.DEVICE_ACTIVE_STATUS_ACTIVE);
                hwDevice.setActiveTime(new Date());
            }

            hwDeviceMapper.updateHwDevice(hwDevice);

            if (connectStatus.equals(HwDictConstants.DEVICE_ONLINE_STATUS_OFFLINE)) {
                this.checkOfflineAlarm(hwDevice);
            }

        }
    }


    /**
     * @param: device
     * @description 离线报警处理
     * @author xins
     * @date 2023-11-14 16:01
     */
    private void checkOfflineAlarm(HwDevice device) {
        Long deviceId = device.getDeviceId();
        Long sceneId = device.getSceneId();

        HwOfflineRule offlineRule = hwOfflineRuleMapper.selectHwOfflineRuleByDeviceId(deviceId);
        if (offlineRule == null) {
            offlineRule = hwOfflineRuleMapper.selectHwOfflineRuleBySceneId(sceneId);
        }

        if (offlineRule != null) {
            Long offlineNumberTime = offlineRule.getOfflineNumberTime();//分钟
            Long offlineNumber = offlineRule.getOfflineNumber();

            if (offlineNumber > 1) {
                Long currentTime = System.currentTimeMillis();
                Long startTime = currentTime - offlineNumberTime * 60 * 1000;
                DeviceStatus queryDeviceStatus = new DeviceStatus();
                queryDeviceStatus.setDeviceId(deviceId);
                queryDeviceStatus.setStartTime(startTime);
                queryDeviceStatus.setEndTime(currentTime);
                queryDeviceStatus.setOnlineStatus(0);

                R<List<Map<String, Object>>> deviceStatusListR = this.remoteTdEngineService.getDeviceStatusList(queryDeviceStatus, SecurityConstants.INNER);
                List<Map<String, Object>> deviceStatusList = deviceStatusListR.getData();
                //在几分钟时间内离线几次的报警信息保存
                if (deviceStatusList != null && deviceStatusList.size() > offlineNumber) {
                    insertHwAlarmInfo(device, offlineRule, deviceId);
                }
            } else {//只要离线就报警
                insertHwAlarmInfo(device, offlineRule, deviceId);
            }
        }
    }

    private void insertHwAlarmInfo(HwDevice device, HwOfflineRule offlineRule, Long deviceId) {
        Date currentDate = new Date();

        HwAlarmInfo alarmInfo = new HwAlarmInfo();
        alarmInfo.setAlarmInfoType(HwDictConstants.ALARM_INFO_TYPE_OFFLINE);
        alarmInfo.setAlarmReleatedId(offlineRule.getOfflineRuleId());
        alarmInfo.setDeviceId(deviceId);
        alarmInfo.setTenantId(device.getTenantId());
        alarmInfo.setSceneId(device.getSceneId());
        alarmInfo.setMonitorUnitId(device.getMonitorUnitId());
        alarmInfo.setAlarmLevelId(offlineRule.getAlarmLevelId());
        alarmInfo.setHandleStatus(HwDictConstants.ALARM_HANDLE_STATUS_NO);
        alarmInfo.setAlarmTime(currentDate);
        alarmInfo.setCreateTime(currentDate);
        hwAlarmInfoMapper.insertHwAlarmInfo(alarmInfo);
        this.handleAlarmLink(offlineRule, device.getDeviceCode());
    }

    /**
     * @param: offlineRule
     * @param: deviceCode
     * @description 处理报警规则联动设备
     * @author xins
     * @date 2023-11-07 16:39
     */
    private void handleAlarmLink(HwOfflineRule offlineRule, String deviceCode) {
        String controlCommandTopic = StringUtils
                .format(HwDictConstants.CONTROL_COMMAND_TOPIC_VALUE, deviceCode);
        if (offlineRule.getLinkFlag().equals(HwDictConstants.ALARM_RULE_LINK_FLAG_YES)) {
            List<HwAlarmRuleLink> offlineRuleLinks = offlineRule.getHwAlarmRuleLinkList();
            offlineRuleLinks.forEach(offlineRuleLink -> {
                this.publishControlCommand(controlCommandTopic, deviceCode, offlineRuleLink.getLinkDeviceFunctionIdentifier());
            });
        }
    }
}
