package com.ruoyi.file.service;

import com.ruoyi.file.utils.FileUploadUtils;
import com.ruoyi.system.api.domain.Base64FileVo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

/**
 * 本地文件存储
 *
 * @author ruoyi
 */
@Primary
@Service
public class LocalSysFileServiceImpl implements ISysFileService {
    /**
     * 资源映射路径 前缀
     */
    @Value("${file.prefix}")
    public String localFilePrefix;

    /**
     * 域名或本机访问地址
     */
    @Value("${file.domain}")
    public String domain;

    /**
     * 上传文件存储在本地的根路径
     */
    @Value("${file.path}")
    private String localFilePath;

    /**
     * 本地文件上传接口
     *
     * @param file 上传的文件
     * @return 访问地址
     * @throws Exception
     */
    @Override
    public String uploadFile(MultipartFile file) throws Exception {
        String name = FileUploadUtils.upload(localFilePath, file);
        String url = domain + localFilePrefix + name;
        return url;
    }


    /**
     * @return String
     * @param: base64FileVo
     * @description 将base64图片转为文件图片保存
     * @author xins
     * @date 2023-10-07 9:22
     */
    @Override
    public String uploadBase64File(Base64FileVo base64FileVo) throws Exception {
        String base64Str = base64FileVo.getBase64Str();
        String imagePath = base64FileVo.getImagePath();
        String extension = base64FileVo.getExtension();

        String name = FileUploadUtils.convertBase64ToImage(localFilePath, base64Str, imagePath, extension);
        String url = domain + localFilePrefix + name;
        return url;
    }
}
